import datetime

from lpworkitems.models import (
    Blueprint,
    BlueprintGroup,
    Complexity,
    Meta,
    Milestone,
    Person,
    TeamParticipation,
    TeamStructure,
    Workitem,
    )


class Factory(object):

    def __init__(self, store, testcase):
        self.store = store
        self.testcase = testcase

    def getUniqueInteger(self):
        return self.testcase.getUniqueInteger()

    def getUniqueStr(self, prefix=None):
        if prefix is None:
            prefix = "string"
        candidate = prefix + str(self.getUniqueInteger())
        return candidate

    def getUniqueUnicode(self, prefix=None):
        candidate = self.getUniqueStr(prefix=prefix)
        return candidate.decode("utf-8")

    def getUniqueDate(self):
        year = self.getUniqueInteger() + 1900
        month = (self.getUniqueInteger() % 11) + 1
        day = (self.getUniqueInteger() % 27) + 1
        return datetime.date(year, month, day)

    def make_milestone(self, name=None, due_date=None,
                       project=None, store=True):
        milestone = Milestone()
        if name is None:
            name = self.getUniqueUnicode(prefix="milestone")
        if due_date is None:
            due_date = self.getUniqueDate()
        milestone.due_date = due_date
        milestone.name = name
        milestone.project = project
        if store:
            self.store.add(milestone)
        return milestone

    def make_blueprint(self, name=None, url=None, status=None,
                       milestone_name=None, definition=None,
                       implementation=None, assignee_name=None,
                       store=True):
        blueprint = Blueprint()
        if name is None:
            name = self.getUniqueUnicode(prefix="blueprint")
        if url is None:
            url = self.getUniqueUnicode(prefix=name+"_url")
        if status is None:
            status = self.getUniqueUnicode(prefix=name+"_status")
        blueprint.name = name
        blueprint.url = url
        blueprint.status = status
        blueprint.milestone_name = milestone_name
        blueprint.definition = definition
        blueprint.implementation = implementation
        blueprint.assignee_name = assignee_name
        if store:
            self.store.add(blueprint)
        return blueprint

    def make_blueprint_group(self, name=None, url=None, status=None,
                             milestone_name=None, implementation=None,
                             store=True):
        blueprint_group = BlueprintGroup()
        if name is None:
            name = self.getUniqueUnicode(prefix="blueprint")
        if url is None:
            url = self.getUniqueUnicode(prefix=name+"_url")
        if status is None:
            status = self.getUniqueUnicode(prefix=name+"_status")
        blueprint_group.name = name
        blueprint_group.url = url
        blueprint_group.status = status
        blueprint_group.milestone_name = milestone_name
        blueprint_group.implementation = implementation
        if store:
            self.store.add(blueprint_group)
        return blueprint_group

    def make_workitem(self, description=None, status=None, date=None,
                      store=True):
        if description is None:
            description = self.getUniqueUnicode(prefix="description")
        if status is None:
            status = Workitem.STATUS_BLOCKED
        if date is None:
            date = self.getUniqueDate()
        workitem = Workitem()
        workitem.description = description
        workitem.status = status
        workitem.date = date
        if store:
            self.store.add(workitem)
        return workitem

    def make_meta(self, store=True):
        meta = Meta()
        if store:
            self.store.add(meta)
        return meta

    def make_complexity(self, store=True):
        complexity = Complexity()
        if store:
            self.store.add(complexity)
        return complexity

    def make_team_participation(self, person_name=None, team_name=None,
                                store=True):
        if person_name is None:
            person_name = self.getUniqueUnicode(prefix="person")
        if team_name is None:
            team_name = self.getUniqueUnicode(prefix="team")
        team_participation = TeamParticipation()
        team_participation.person_name = person_name
        team_participation.team_name = team_name
        if store:
            self.store.add(team_participation)
        return team_participation

    def make_team_structure(self, subteam_name=None, superteam_name=None,
                            store=True):
        if subteam_name is None:
            subteam_name = self.getUniqueUnicode(prefix="subteam")
        if superteam_name is None:
            superteam_name = self.getUniqueUnicode(prefix="superteam")
        team_structure = TeamStructure()
        team_structure.subteam_name = subteam_name
        team_structure.superteam_name = superteam_name
        if store:
            self.store.add(team_structure)
        return team_structure

    def make_person(self, name=None, store=True):
        if name is None:
            name = self.getUniqueUnicode(prefix="person")
        person = Person()
        person.name = name
        if store:
            self.store.add(person)
        return person
