class LaunchpadAPIObject(object):

    RESOURCE_TYPE = None

    def __init__(self):
        self.self_link = None
        self.web_link = None

    @property
    def resource_type_link(self):
        return "some_url#%s" % self.RESOURCE_TYPE


class LaunchpadMilestone(LaunchpadAPIObject):

    RESOURCE_TYPE = "milestone"

    def __init__(self, name, date_targeted, target):
        super(LaunchpadMilestone, self).__init__()
        self.name = name
        self.date_targeted = date_targeted
        self.target = target


class LaunchpadProject(LaunchpadAPIObject):

    RESOURCE_TYPE = "project"

    def __init__(self, name):
        super(LaunchpadProject, self).__init__()
        self.name = name


class LaunchpadProjectSeries(LaunchpadAPIObject):

    RESOURCE_TYPE = "project_series"

    def __init__(self):
        super(LaunchpadProjectSeries, self).__init__()
        self.name = None
        self.project = None


class LaunchpadBlueprint(LaunchpadAPIObject):

    RESOURCE_TYPE = "blueprint"

    def __init__(self):
        super(LaunchpadBlueprint, self).__init__()
        self.priority = None
        self.definition_status = None
        self.implementation_status = None
        self.approver = None
        self.drafter = None
        self.assignee = None
        self.milestone = None
        self.summary = None
        self.whiteboard = None
        self.specification_url = None


class LaunchpadPerson(LaunchpadAPIObject):

    RESOURCE_TYPE = "person"

    def __init__(self):
        super(LaunchpadPerson, self).__init__()
        self.name = None


class LaunchpadBug(LaunchpadAPIObject):

    RESOURCE_TYPE = "bug"

    def __init__(self):
        super(LaunchpadBug, self).__init__()
        self.id = None
        self.title = None
        self.bug_tasks = []


class LaunchpadBugTask(LaunchpadAPIObject):

    RESOURCE_TYPE = "bug_task"

    def __init__(self):
        super(LaunchpadBugTask, self).__init__()
        self.status = None
        self.target = None
        self.bug = None
        self.assignee = None
        self.milestone = None
        self.related_tasks = []


class LaunchpadDistribution(LaunchpadAPIObject):

    RESOURCE_TYPE = "distribution"

    def __init__(self):
        super(LaunchpadDistribution, self).__init__()
        self.name = None


class LaunchpadDistributionSeries(LaunchpadAPIObject):

    RESOURCE_TYPE = "distro_series"

    def __init__(self):
        super(LaunchpadDistributionSeries, self).__init__()
        self.name = None
        self.distribution = None


class LaunchpadDistributionSourcePackage(LaunchpadAPIObject):
    """A source package in a distribution (not series-specific.)"""

    RESOURCE_TYPE = "distribution_source_package"

    def __init__(self):
        super(LaunchpadDistributionSourcePackage, self).__init__()
        self.distribution = None
        self.name = None


class LaunchpadSourcePackage(LaunchpadAPIObject):
    """A source in a particular release of a distribution."""

    RESOURCE_TYPE = "source_package"

    def __init__(self):
        super(LaunchpadSourcePackage, self).__init__()
        self.distroseries = None
        self.name = None

    @property
    def distribution(self):
        return self.distroseries.distribution


class FakeLaunchpad(object):

    def __init__(self, factory):
        self._factory = factory
        self._urls = {}
        self.people = {}

    def load(self, url):
        assert url in self._urls, "Loading unknown URL: %s" % url
        return self._urls[url]

    def set_load_result(self, url, result):
        self._urls[url] = result

    def register_object(self, obj):
        if obj.self_link is None:
            obj.self_link = self.make_self_link()
        if obj.web_link is None:
            obj.web_link = self.make_web_link()
        self.set_load_result(obj.self_link, obj)

    def make_self_link(self):
        return self._factory.getUniqueUnicode(prefix="self_link")

    def make_web_link(self):
        return self._factory.getUniqueUnicode(prefix="web_link")

    def make_project(self, name=None):
        if name is None:
            name = self._factory.getUniqueUnicode(prefix="lpproject")
        project = LaunchpadProject(name)
        self.register_object(project)
        return project

    def make_project_series(self, name=None, project=None):
        if project is None:
            project = self.make_project()
        if name is None:
            name = self._factory.getUniqueUnicode(
                prefix="%s_series" % project.name)
        project_series = LaunchpadProjectSeries()
        project_series.name = name
        project_series.project = project
        self.register_object(project_series)
        return project_series

    def make_milestone(self, name=None, date_targeted=None, target=None):
        if name is None:
            name = self._factory.getUniqueUnicode(prefix="lpmilestone")
        if target is None:
            target = self.make_project()
        milestone = LaunchpadMilestone(name, date_targeted, target)
        self.register_object(milestone)
        return milestone

    def make_blueprint(self, name=None, priority=None,
                       definition_status=None,
                       implementation_status=None, approver=None,
                       drafter=None, assignee=None, milestone=None,
                       whiteboard=None, specification_url=None,
                       summary=None):
        if name is None:
            name = self._factory.getUniqueUnicode(prefix="lpblueprint")
        blueprint = LaunchpadBlueprint()
        blueprint.name = name
        blueprint.priority = priority
        blueprint.definition_status = definition_status
        blueprint.implementation_status = implementation_status
        blueprint.approver = approver
        blueprint.drafter = drafter
        blueprint.assignee = assignee
        blueprint.milestone = milestone
        blueprint.whiteboard = whiteboard
        blueprint.specification_url = specification_url
        blueprint.summary = summary
        self.register_object(blueprint)
        return blueprint

    def make_person(self, name=None):
        if name is None:
            name = self._factory.getUniqueUnicode(prefix="lpperson")
        person = LaunchpadPerson()
        person.name = name
        self.register_object(person)
        self.people[name] = person
        return person

    def make_bug(self, id_=None, title=None):
        if id_ is None:
            id_ = self._factory.getUniqueInteger()
        if title is None:
            title = self._factory.getUniqueUnicode(
                prefix="bug_%d_description" % id_)
        bug = LaunchpadBug()
        bug.id = id_
        bug.title = title
        self.register_object(bug)
        return bug

    def make_bug_task(self, bug=None, status=None, target=None,
                      assignee=None, milestone=None):
        if bug is None:
            bug = self.make_bug()
        if target is None:
            target = self.make_project()
        if status is None:
            status = u"Expired"
        bug_task = LaunchpadBugTask()
        bug_task.bug = bug
        bug_task.status = status
        bug_task.target = target
        bug_task.assignee = assignee
        bug_task.milestone = milestone
        self.register_object(bug_task)
        bug.bug_tasks.append(bug_task)
        return bug_task

    def make_distribution(self, name=None):
        if name is None:
            name = self._factory.getUniqueUnicode(prefix="lpdistribution")
        distribution = LaunchpadDistribution()
        distribution.name = name
        self.register_object(distribution)
        return distribution

    def make_distro_series(self, distribution=None, name=None):
        if distribution is None:
            distribution = self.make_distribution()
        if name is None:
            name = self._factory.getUniqueUnicode(
                prefix="%s_release" % distribution.name)
        distro_series = LaunchpadDistributionSeries()
        distro_series.name = name
        distro_series.distribution = distribution
        self.register_object(distro_series)
        return distro_series

    def make_distribution_source_package(self, distribution=None, name=None):
        if distribution is None:
            distribution = self.make_distribution()
        if name is None:
            name = self._factory.getUniqueUnicode(
                prefix="%s_package" % distribution.name)
        distribution_source_package = LaunchpadDistributionSourcePackage()
        distribution_source_package.distribution = distribution
        distribution_source_package.name = name
        self.register_object(distribution_source_package)
        return distribution_source_package

    def make_source_package(self, distroseries=None, name=None):
        if distroseries is None:
            distroseries = self.make_distro_series()
        if name is None:
            name = self._factory.getUniqueUnicode(
                prefix="%s_package" % distroseries.name)
        source_package = LaunchpadSourcePackage()
        source_package.distroseries = distroseries
        source_package.name = name
        self.register_object(source_package)
        return source_package
