import datetime
from testtools import TestCase
import report_tools


class FillTemplateTests(TestCase):

    def test_finds_template(self):
        output = report_tools.fill_template("test.html", dict(data="foo"))
        self.assertEqual("Testing foo\n", output)


class WeekdaysBetweenTests(TestCase):

    def test_start_after_end(self):
        start = datetime.date(2011, 02, 17)
        end = datetime.date(2011, 02, 16)
        self.assertEqual(0, report_tools.weekdays_between(start, end))

    def test_start_equals_end(self):
        start = datetime.date(2011, 02, 16)
        end = datetime.date(2011, 02, 16)
        self.assertEqual(0, report_tools.weekdays_between(start, end))

    def test_one_day_before_thursday(self):
        # Wednesday
        start = datetime.date(2011, 02, 16)
        # Thursday
        end = datetime.date(2011, 02, 17)
        self.assertEqual(1, report_tools.weekdays_between(start, end))

    def test_one_day_before_saturday(self):
        # Friday
        start = datetime.date(2011, 02, 18)
        # Saturday
        end = datetime.date(2011, 02, 19)
        self.assertEqual(1, report_tools.weekdays_between(start, end))

    def test_one_day_before_sunday(self):
        # Saturday
        start = datetime.date(2011, 02, 19)
        # Sunday
        end = datetime.date(2011, 02, 20)
        self.assertEqual(0, report_tools.weekdays_between(start, end))

    def test_one_day_before_monday(self):
        # Sunday
        start = datetime.date(2011, 02, 20)
        # Monday
        end = datetime.date(2011, 02, 21)
        self.assertEqual(0, report_tools.weekdays_between(start, end))

    def test_two_days_before_monday(self):
        # Saturday
        start = datetime.date(2011, 02, 19)
        # Monday
        end = datetime.date(2011, 02, 21)
        self.assertEqual(0, report_tools.weekdays_between(start, end))

    def test_three_days_before_monday(self):
        # Friday
        start = datetime.date(2011, 02, 18)
        # Monday
        end = datetime.date(2011, 02, 21)
        self.assertEqual(1, report_tools.weekdays_between(start, end))

    def test_one_day_before_tuesday(self):
        # Monday
        start = datetime.date(2011, 02, 21)
        # Tuesday
        end = datetime.date(2011, 02, 22)
        self.assertEqual(1, report_tools.weekdays_between(start, end))

    def test_sunday_to_saturday(self):
        # Sunday
        start = datetime.date(2011, 02, 13)
        # Saturday
        end = datetime.date(2011, 02, 19)
        self.assertEqual(5, report_tools.weekdays_between(start, end))

    def test_saturday_to_saturday(self):
        # Saturday
        start = datetime.date(2011, 02, 12)
        # Next Saturday
        end = datetime.date(2011, 02, 19)
        self.assertEqual(5, report_tools.weekdays_between(start, end))

    def test_monday_to_saturday(self):
        # Monday
        start = datetime.date(2011, 02, 14)
        # Saturday
        end = datetime.date(2011, 02, 19)
        self.assertEqual(5, report_tools.weekdays_between(start, end))

    def test_thursday_to_tuesday(self):
        # Thursday
        start = datetime.date(2011, 02, 17)
        # Next tuesday
        end = datetime.date(2011, 02, 22)
        # 3 = Thursday, Friday, Monday
        self.assertEqual(3, report_tools.weekdays_between(start, end))

    def test_thirteen_days_to_tuesday(self):
        # Wednesday
        start = datetime.date(2011, 02, 9)
        # Next, next tuesday
        end = datetime.date(2011, 02, 22)
        # 4 for Wednesday, Thursday, Friday, Monday
        # + 5 for the whole week
        # = 9
        self.assertEqual(9, report_tools.weekdays_between(start, end))


class DurationElapsedTests(TestCase):

    def test_start_date_after_end_date(self):
        start = datetime.date(2011, 02, 9)
        end = datetime.date(2011, 02, 8)
        current = end
        self.assertEqual(
            100, report_tools.duration_elapsed(start, end, current))

    def test_start_date_equals_end_date(self):
        start = datetime.date(2011, 02, 9)
        end = start
        current = end
        self.assertEqual(
            100, report_tools.duration_elapsed(start, end, current))

    def test_current_date_after_end_date(self):
        start = datetime.date(2011, 02, 9)
        end = datetime.date(2011, 02, 10)
        current = datetime.date(2011, 02, 11)
        self.assertEqual(
            100, report_tools.duration_elapsed(start, end, current))

    def test_current_date_equals_end_date(self):
        start = datetime.date(2011, 02, 9)
        end = datetime.date(2011, 02, 10)
        current = end
        self.assertEqual(
            100, report_tools.duration_elapsed(start, end, current))

    def test_start_date_after_current_date(self):
        start = datetime.date(2011, 02, 9)
        end = datetime.date(2011, 02, 10)
        current = datetime.date(2011, 02, 8)
        self.assertEqual(
            0, report_tools.duration_elapsed(start, end, current))

    def test_start_date_equals_current_date(self):
        start = datetime.date(2011, 02, 9)
        end = datetime.date(2011, 02, 10)
        current = start
        self.assertEqual(
            0, report_tools.duration_elapsed(start, end, current))

    def test_percentage_computed(self):
        start = datetime.date(2011, 02, 9)
        end = datetime.date(2011, 02, 11)
        current = datetime.date(2011, 02, 10)
        self.assertEqual(
            50, report_tools.duration_elapsed(start, end, current))


def test_suite():
    from unittest import TestLoader
    loader = TestLoader()
    suite = loader.loadTestsFromName(__name__)
    suite.addTests(loader.loadTestsFromName("lpworkitems.tests.test_collect"))
    suite.addTests(loader.loadTestsFromName("lpworkitems.tests.test_error_collector"))
    suite.addTests(loader.loadTestsFromName("lpworkitems.tests.test_factory"))
    suite.addTests(loader.loadTestsFromName("lpworkitems.tests.test_fake_launchpad"))
    suite.addTests(loader.loadTestsFromName("lpworkitems.tests.test_models"))
    return suite
